import { Component, OnInit, AfterViewInit, OnDestroy, ViewChild, ElementRef, ChangeDetectorRef } from '@angular/core';
import { NativeInputComponent } from '../../_abstract/native-input.component';
import { FriendlyValidationErrorsService } from './../../../services/friendly-validation-errors.service';

import { Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, tap } from 'rxjs/operators';


@Component({
	selector: 'app-checkbox',
	templateUrl: './clr-checkbox.component.html',
	styleUrls: ['./clr-checkbox.component.scss']
})
export class ClrCheckboxComponent extends NativeInputComponent implements OnInit, AfterViewInit, OnDestroy {

	@ViewChild('labelRef', { static: false })
	labelRef: ElementRef;

	exposeMetaInTemplate: string[] = ['label', 'editable'];

	label: string;
	editable: boolean;

	label$: Subject<string> = new Subject();
	labelSBX: Subscription;
	labelTxt: string = '';

	readonly componentName = 'CheckboxComponent'; // For AOT compatibility, as class names don't survive minification

	constructor(
		protected valErrsService: FriendlyValidationErrorsService,
		protected _cdr: ChangeDetectorRef
	) {
		super(valErrsService, _cdr);
	}

	ngOnInit() {
		super.ngOnInit();
		if (this._meta.editable) {
			this.labelSBX = this.label$.pipe(
				debounceTime(500),
				distinctUntilChanged(),
				tap(label => this.labelTxt = label)
			).subscribe(label => {
				this.setValue(!!this.control.value, label);
			});			
		}
	}

	ngAfterViewInit() {
		if (this.control.value && this._meta.transformer?.inputFn) {
			const boundFn = this._meta.transformer.inputFn.bind(this);
			try {
				boundFn(this.control.value);
			} catch(e) {
				console.error('Bad Transformer InputFn');
				console.error(e);
			}
		}
	}

	ngOnDestroy() {
		if (this.labelSBX) {
			this.labelSBX.unsubscribe();
		}
	}

	checkedChange(cb: HTMLInputElement) {
		this.setValue(cb.checked);
		this.handleChange();
	}

	setValue(checked: boolean, labelTxt: string = this.labelTxt) {
		let valToSet = checked ? this._meta.checkedValue : false;
		if (checked && typeof this._meta.transformer?.outputFn === 'function') {
			try {
				valToSet = this._meta.transformer.outputFn(this._meta.checkedValue, this.labelTxt);
			} catch(e) {
				console.error('Bad Transformer OutputFn');
				console.error(e);
			}
		}
		this.control.setValue(valToSet);
	}

	focusEditable(e: MouseEvent): void {
		setTimeout(() => this.labelRef.nativeElement.focus(), 250);
		e.preventDefault();
	}

	defocusEditable(): void {
		this.labelRef.nativeElement.blur();
	}
	
	labelEdited(): void {
		const label = (this.labelRef.nativeElement) as HTMLSpanElement;
		this.label$.next(label.textContent.trim());
	}

}
